-- Original version: Simon Thompson

module MakeTree ( makeTree, toTreeList )  -- [(Char,Int)] -> HTree
      where

import HTypes ( HTree ( Leaf, Node ), Bit ( L, R ), HCode, HTable )

makeTree :: [(Char,Int)] -> HTree
makeTree = makeCodes . toTreeList

toTreeList ::  [(Char,Int)] -> [HTree]
toTreeList = map (uncurry Leaf)

-- uncurry converts a curried function to a function on pairs
-- (a -> b -> c) -> (a,b) -> c

-- curry converts an uncurried function to a curried function
-- ((a,b) -> c) -> a -> b -> c

makeCodes :: [HTree] -> HTree
makeCodes [t] = t
makeCodes ts = makeCodes (amalgamate ts)

amalgamate :: [HTree] -> [HTree]
amalgamate (t1:t2:ts) = insertTree (pair t1 t2) ts

pair :: HTree -> HTree -> HTree
pair t1 t2 = Node (f1+f2) t1 t2
             where
             f1 = value t1
             f2 = value t2

value :: HTree -> Int
value (Leaf _ n) = n
value (Node n _ _) = n

insertTree ::HTree -> [HTree] -> [HTree]
insertTree t [] = [t]
insertTree t (t1:ts) | value t < value t1 = t:t1:ts
                     | otherwise = t1 : insertTree t ts


